import java.io.*;

public class RuKuIntegrator extends Integrator
{
     public double[] k1;
     public double[] k2;
     public double[] k3;
     public double[] k4;
     public double[] yh;

     public RuKuIntegrator(Modell modell)
     {
         super(modell);
         k1     = new double[modell.holeAnzahlGleichungen()];
         k2     = new double[modell.holeAnzahlGleichungen()];
         k3     = new double[modell.holeAnzahlGleichungen()];
         k4     = new double[modell.holeAnzahlGleichungen()];
         yh     = new double[modell.holeAnzahlGleichungen()];
         
     }

     public void zeitschritt(double dt)
     {
         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             y_alt[i] = y[i];

         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             k1[i] = modell.rechteSeite(i,y_alt,t); 

         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             yh[i] = y_alt[i]+0.5*dt*k1[i]; 

         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             k2[i] = modell.rechteSeite(i,yh,t+0.5*dt); 

         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             yh[i] = y_alt[i]+0.5*dt*k2[i]; 

         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             k3[i] = modell.rechteSeite(i,yh,t+0.5*dt);                  

         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             yh[i] = y_alt[i]+dt*k3[i]; 

         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             k4[i] = modell.rechteSeite(i,yh,t+dt);   
             
         for(int i=0;i<modell.holeAnzahlGleichungen();i++)
             y[i] = y_alt[i] + (dt/6.0)*(k1[i] + 2.0*k2[i]+2.0*k3[i]+k4[i]);
                                   
         t+=dt;
     }
     
     public static void main(String[] args)
     {
         if(args.length!=3)
         {
             System.out.println("Benutzung des Programms:");
             System.out.println("RuKuIntegrator <C> <D> <m>");
             System.exit(0);
         }
    
         double C = Double.parseDouble(args[0]);
         double D = Double.parseDouble(args[1]);
         double m = Double.parseDouble(args[2]);
         SchwingerModell modell = new SchwingerModell(C,D,m);
         RuKuIntegrator integrator = new RuKuIntegrator(modell);
         double[] y = {1.0,0.0};
         integrator.setzeAnfangsbedingungen(y);
                          
         try
         {
             PrintWriter pw = new PrintWriter(new FileWriter("data.txt"));
         
             for(int i=0;i<100;i++)
             {
                 pw.println(integrator.t+" "+integrator.y[0]);
                 System.out.println(integrator.t+" "+integrator.y[0]);
                 integrator.zeitschritt(0.1);
             }

             pw.close();         
         
             Runtime runtime = Runtime.getRuntime();
             runtime.exec("C:/Programme/scilab-4.1/bin/Scilex.exe -e exec(\'zeige.sce\')");
         }
         catch(Exception e)
         {
         }
     }
}
